<?php
/**
 * Airtable API class
 * 
 * @package Page_Generator_Pro
 * @author  Tim Carr
 * @version 3.3.4
 */
class Page_Generator_Pro_Airtable extends Page_Generator_Pro_API {

    /**
     * Holds the API endpoint
     *
     * @since   3.3.4
     *
     * @var     string
     */
    public $api_endpoint = 'https://api.airtable.com/v0';

    /**
     * Holds the flag determining if the request data should be encoded
     * into a JSON string
     *
     * If false, data is encoded using http_build_query()
     *
     * @since   3.3.4
     *
     * @var     bool
     */
    public $is_json_request = false;

    /**
     * Returns data from the given Base ID and Table Name
     *
     * @since   3.3.4
     *
     * @param   string  $api_key        API Key
     * @param   string  $base_id        Base ID
     * @param   string  $table_name     Table Name 
     * @return  mixed                   WP_Error | array
     */
	public function get_data( $api_key, $base_id, $table_name ) {

        // Set API Key
        $this->set_api_key( $api_key );

        // Set Headers
        $this->set_headers( array(
            'Authorization' => 'Bearer ' . $this->api_key,
        ) );

        // Send Request
        $records = $this->response(
            $this->get( $base_id . '/' . rawurlencode( $table_name ) )
        );

        // Return if the response is an error
        if ( is_wp_error( $records ) ) {
            return $records;
        }

        // Convert data into an array compatible with the Keyword system
        $columns = array();
        $data = array();
        foreach ( $records as $index => $record ) {
            $data[ $index ] = array();
            foreach ( $record->fields as $field => $value ) {
                // Build columns
                if ( $index == 0 ) {
                    $columns[ $field ] = $field;
                }

                // Build row
                $data[ $index ][ $field ] = $value;
            }

            // Sort row columns by name, so that the data order is always the same
            ksort( $data[ $index ] );

            // If more than one column exists, implode the data and encapsulate in quotation marks
            if ( count( $columns ) > 1 ) {
                $data[ $index ] = '"' . implode( '","', $data[ $index ] ) . '"';  
            } else {
                $data[ $index ] = implode( ',', $data[ $index ] ); 
            }
        }

        // Sort columns by name, so they match the order of each row's data
        ksort( $columns );

        // Return
        return array(
            'delimiter' => ( count( $columns ) > 1 ? ',' : '' ),
            'columns'   => ( count( $columns ) > 1 ? $columns : '' ),
            'data'      => $data,
        );

	}

    /**
     * Inspects the response from the API call, returning an error
     * or data
     *
     * @since   3.3.4
     *
     * @param   mixed   $response   Response (WP_Error | object)
     * @return  mixed               WP_Error | object
     */
    private function response( $response ) {

        // If the response is an error, return it
        if ( is_wp_error( $response ) ) {
            return new WP_Error(
                'page_generator_pro_airtable_error',
                sprintf( 
                    /* translators: Error message */
                    __( 'Airtable: %s', 'page-generator-pro' ),
                    $response->get_error_message()
                )
            );
        }

        // Return an error if an error is present in the response
        if ( isset( $response->error ) ) {
            // Depending on the error, we might be able to return a more helpful error message
            if ( is_string( $response->error ) ) {
                switch ( $response->error ) {
                    case 'NOT_FOUND':
                        $message = __( 'Base ID or Table Name not found.', 'page-generator-pro' );
                        break;
                    default:
                        $message = $response->error;
                        break;
                }
            } else {
                $message = $response->error->message;
            }

            // Return error object
            return new WP_Error( 
                'page_generator_pro_airtable_error',
                sprintf( 
                    /* translators: Error message */
                    __( 'Airtable: %s', 'page-generator-pro' ),
                    $message
                )
            );   
        }

        // Return records
        return $response->records;

    }

}