<?php
/**
 * GoodLayers Integration
 * 
 * @package Page_Generator_Pro
 * @author  Tim Carr
 * @version 3.3.9
 */
class Page_Generator_Pro_GoodLayers extends Page_Generator_Pro_Integration {

    /**
     * Holds the base object.
     *
     * @since   3.3.9
     *
     * @var     object
     */
    public $base;

    /**
     * Constructor
     * 
     * @since   3.3.9
     *
     * @param   object  $base    Base Plugin Class
     */
    public function __construct( $base ) {

        // Store base class
        $this->base = $base;

        // Set Plugin
        $this->plugin_folder_filename = 'goodlayers-core/goodlayers-core.php';

        // Set Meta Keys used by the GoodLayers Page Builder
        $this->meta_keys = array(
        	'gdlr-core-page-builder',
        );

        // Content Groups: Add Overwrite Section if Yoast SEO enabled
        add_filter( 'page_generator_pro_common_get_content_overwrite_sections', array( $this, 'add_overwrite_section' ) );

        // Content Groups: Ignore meta keys if overwriting is disabled for Page Options
        add_filter( 'page_generator_pro_generate_set_post_meta_ignored_keys', array( $this, 'prevent_post_meta_copy_to_generated_content' ), 10, 4 );

        // Remove Page Builder data from Group Settings if overwriting content is disabled, and an existing generated page already exists
        add_filter( 'page_generator_pro_generate_remove_content_from_settings_ignored_keys', array( $this, 'remove_post_meta_from_content_group' ), 10, 2 );

        // Remove Plugin data from Group Settings if Plugin isn't active on Generation
        add_filter( 'page_generator_pro_groups_get_settings_remove_orphaned_settings', array( $this, 'remove_orphaned_settings' ) );

    }

    /**
     * Defines available content overwrite sections.
     *
     * @since   3.3.9
     *
     * @param   array   $sections    Content Overwrite Sections
     * @return  array                Content Overwrite Sections
     */
    public function add_overwrite_section( $sections ) {

        // Bail if GoodLayers Page Options isn't active
        if ( ! $this->is_active() ) {
            return $sections;
        }

        // Add GoodLayers Page Options
        $sections['goodlayers_page_options'] = __( 'GoodLayers: Page Options', 'page-generator-pro' );

        // Return
        return $sections;

    }

    /**
     * Adds GoodLayers Theme Page Option Meta Keys to the array of excluded Post Meta Keys if GoodLayers Page Options
     * metadata should not be overwritten on Content Generation.
     * 
     * GoodLayers Page Builder data is handled by remove_post_meta_from_content_group()
     *
     * @since   3.3.8
     *
     * @param   array   $ignored_keys   Ignored Keys (preg_match() compatible regex expressions are supported)
     * @param   int     $post_id        Generated Post ID
     * @param   array   $settings       Group Settings
     * @param   array   $post_args      wp_insert_post() / wp_update_post() arguments
     * @return  array   $ignored_keys   Ignored Keys
     */
    public function prevent_post_meta_copy_to_generated_content( $ignored_keys, $post_id, $settings, $post_args ) {

        // Determine if we want to create/replace GoodLayers Page Option metdata
        $overwrite = ( isset( $post_args['ID'] ) && ! array_key_exists( 'goodlayers_page_options', $settings['overwrite_sections'] ) ? false : true ); 

        // If overwriting is enabled, no need to exclude anything
        if ( $overwrite ) {
            return $ignored_keys;
        }

        // Add GoodLayers Page Option Meta Keys so they are not overwritten on the Generated Post
        // We don't read $this->meta_keys as that contains Meta Keys for the Page Builder, not the Page Options section
        return array_merge( $ignored_keys, array(
        	'gdlr-core-page-option',
        	'gdlr-core-post-option',
        ) );

    }
				
    /**
     * Removes orphaned GoodLayers metadata in the Group Settings during Generation,
     * if GoodLayers is not active
     * 
     * @since   3.3.9
     * 
     * @param   array   $settings   Group Settings
     * @return  array               Group Settings
     */
    public function remove_orphaned_settings( $settings ) {

        // Don't remove settings if GoodLayers is active
        if ( $this->is_active() ) {
            return $settings;
        }

        // Remove GoodLayers Meta Keys from the Group Settings during Generation
        // We don't read $this->meta_keys as that contains Meta Keys for the Page Builder, not both the Page Builder and Page Options
        // - and we want to remove both
        return $this->remove_orphaned_settings_metadata( $settings, array(
            "/^gdlr-(.*)/i",
            "/^gdlr_(.*)/i",
        ) );

    }

}