<?php
/**
 * ListingPro class
 * 
 * @package  Page Generator Pro
 * @author   Tim Carr
 * @version  2.8.6
 */
class Page_Generator_Pro_ListingPro extends Page_Generator_Pro_Integration {

    /**
     * Holds the base object.
     *
     * @since   2.8.6
     *
     * @var     object
     */
    public $base;

    /**
     * Constructor
     * 
     * @since   2.8.6
     *
     * @param   object $base    Base Plugin Class
     */
    public function __construct( $base ) {

        // Store base class
        $this->base = $base;

        // Set Plugin
        $this->plugin_folder_filename = 'listingpro-plugin/plugin.php';

        // Set Meta Keys used by this Plugin
        $this->meta_keys = array(
            'lp_listingpro_options',
            'lp_listingpro_options_fields',
        );

        // Add Overwrite Section if ListingPro enabled
        add_filter( 'page_generator_pro_common_get_content_overwrite_sections', array( $this, 'add_overwrite_section' ) );

        // Ignore ListingPro meta keys if overwriting is disabled for ListingPro
        add_filter( 'page_generator_pro_generate_set_post_meta_ignored_keys', array( $this, 'prevent_post_meta_copy_to_generated_content' ), 10, 4 );

        // Remove Plugin data from Group Settings if Plugin isn't active on Generation
        add_filter( 'page_generator_pro_groups_get_settings_remove_orphaned_settings', array( $this, 'remove_orphaned_settings' ) );

        add_filter( 'page_generator_pro_common_get_excluded_post_types', array( $this, 'exclude_post_types' ) );
        add_filter( 'page_generator_pro_groups_ui_get_post_type_conditional_metaboxes', array( $this, 'get_post_type_conditional_metaboxes' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'listings_enqueue_scripts' ) );
        add_action( 'page_generator_pro_groups_ui_add_meta_boxes', array( $this, 'listings_add_meta_boxes' ) );
        add_action( 'page_generator_pro_groups_save', array( $this, 'save' ), 10, 3 );
        add_action( 'page_generator_pro_groups_save', array( $this, 'save_event' ), 10, 3 );

    }

    /**
     * Defines available content overwrite sections.
     *
     * @since   2.9.0
     *
     * @param   array   $sections    Content Overwrite Sections
     * @return  array                Content Overwrite Sections
     */
    public function add_overwrite_section( $sections ) {

        // Bail if ListingPro isn't active
        if ( ! $this->is_active() ) {
            return $sections;
        }

        // Add ListingPro
        $sections['listingpro'] = __( 'ListingPro', 'page-generator-pro' );

        // Return
        return $sections;

    }

    /**
     * Adds ListingPro Post Meta Keys to the array of excluded Post Meta Keys if ListingPro
     * metadata should not be overwritten on Content Generation
     *
     * @since   2.9.0
     *
     * @param   array   $ignored_keys   Ignored Keys (preg_match() compatible regex expressions are supported)
     * @param   int     $post_id        Generated Post ID
     * @param   array   $settings       Group Settings
     * @param   array   $post_args      wp_insert_post() / wp_update_post() arguments
     * @return  array   $ignored_keys   Ignored Keys
     */
    public function prevent_post_meta_copy_to_generated_content( $ignored_keys, $post_id, $settings, $post_args ) {

        // Determine if we want to create/replace ListingPro metdata
        $overwrite = ( isset( $post_args['ID'] ) && ! array_key_exists( 'listingpro', $settings['overwrite_sections'] ) ? false : true ); 

        // If overwriting is enabled, no need to exclude anything
        if ( $overwrite ) {
            return $ignored_keys;
        }

        // Add ListingPro Meta Keys so they are not overwritten on the Generated Post
        return array_merge( $ignored_keys, $this->meta_keys );

    }

    /**
     * Removes orphaned metadata in the Group Settings during Generation,
     * if ListingPro is not active
     * 
     * @since   3.3.7
     * 
     * @param   array   $settings   Group Settings
     * @return  array               Group Settings
     */
    public function remove_orphaned_settings( $settings ) {

        // Don't remove settings if ListingPro is active
        if ( $this->is_active() ) {
            return $settings;
        }

        // Remove ListingPro Meta Keys from the Group Settings during Generation
        return $this->remove_orphaned_settings_metadata( $settings, $this->meta_keys );

    }

    /**
     * Excludes some ListingPro Post Types that we don't support generation for
     *
     * @since   2.8.6
     *
     * @param   array   $post_types     Excluded Post Types
     * @return  array                   Excluded Post Types
     */
    public function exclude_post_types( $post_types ) {

        // Bail if ListingPro isn't active
        if ( ! function_exists( 'is_plugin_active' ) || ! is_plugin_active( 'listingpro-plugin/plugin.php' ) ) {
            return $post_types;
        }

        return array_merge( $post_types, array(
            'lp-ads',
            'lp-claims',
        ) );

    }

    /**
     * Define metaboxes that should only display based on the value of Publish > Post Type
     * in the Content Groups UI.
     *
     * @since   2.8.6
     *
     * @param   array   $metaboxes  Metabox ID Keys and Post Type Values array
     * @return  array               Metabox ID Keys and Post Type Values array
     */
    public function get_post_type_conditional_metaboxes( $metaboxes ) {

        return array_merge( $metaboxes, array(
            'listing_meta_settings' => array(
                'listing',
            ),
            'Reviews_meta_settings' => array(
                'lp-reviews',
            ),
            'event_meta_box' => array(
                'events',
            )
        ) );

    }

    /**
     * Enqueue ListingPro scripts to Content Groups for the Listings Post Type
     *
     * @since   2.8.6
     */
    public function listings_enqueue_scripts() {

        // Bail if ListingPro isn't active
        if ( ! function_exists( 'is_plugin_active' ) || ! is_plugin_active( 'listingpro-plugin/plugin.php' ) ) {
            return;
        }

        wp_enqueue_script( 'accorianui', plugins_url( 'assets/js/jqueryuiaccordian.js', dirname( dirname( __FILE__ ) ) ), 'jquery', '', true );
        wp_enqueue_script( 'jquery-ui', get_template_directory_uri() . '/assets/js/jquery-ui.js', 'jquery', '', true );
        wp_enqueue_script( 'jquery-ui-trigger', get_template_directory_uri() . '/assets/js/jquery-ui-trigger.js', 'jquery', '', true );
        wp_localize_script( 'jquery-ui-trigger','global', array( 'ajax' => admin_url( 'admin-ajax.php' ) ) );
        wp_enqueue_script( 'jquery-droppin', get_template_directory_uri() . '/assets/js/drop-pin.js', 'jquery', '', true );

    }

    /**
     * Adds ListingPro Meta Boxes to Content Groups for the Listings Post Type
     *
     * @since   2.8.6
     */
    public function listings_add_meta_boxes() {

        global $listingpro_settings, $reviews_options;

        // Bail if ListingPro isn't active
        if ( ! function_exists( 'is_plugin_active' ) || ! is_plugin_active( 'listingpro-plugin/plugin.php' ) ) {
            return;
        }

        // Listing
        add_meta_box( 
            'listing_meta_settings',
            esc_html__( 'listing settings', 'listingpro-plugin' ),
            'listingpro_metabox_render',
            $this->base->get_class( 'post_type' )->post_type_name,
            'normal',
            'high',
            $listingpro_settings
        );

        // Review
        add_meta_box(
            'Reviews_meta_settings',
            esc_html__( 'Additional Setting', 'listingpro-plugin' ),
            'reviews_metabox_render',
            $this->base->get_class( 'post_type' )->post_type_name,
            'normal',
            'high',
            $reviews_options
        );

        // Event
        add_meta_box( 
            'event_meta_box', 
            __( 'Event Details', 'listingpro' ), 
            'event_meta_box', 
            $this->base->get_class( 'post_type' )->post_type_name,
            'normal',
            'high'
        );

    }

    /**
     * Saves ListingPro metadata from any metaboxes displayed on the Content Group,
     * emulating ListingPro's savePostMeta() function
     *
     * @since   2.8.6
     *
     * @param   int     $group_id   Group ID
     * @param   array   $settings   Group Settings
     * @param   array   $request    $_REQUEST data, unsanitized
     */
    public function save( $group_id, $settings, $request ) {

        global $listingpro_settings, $reviews_options, $listingpro_formFields, $claim_options, $ads_options, $page_options, $post_options, $price_plans_options;

        // Bail if ListingPro isn't active
        if ( ! function_exists( 'is_plugin_active' ) || ! is_plugin_active( 'listingpro-plugin/plugin.php' ) ) {
            return;
        }

        // Verify nonce
        // @TODO
        /*
        if ( ! isset( $request['lp_meta_box_nonce'] ) ) {
            return;
        }
        if ( ! wp_verify_nonce( $request['lp_meta_box_nonce'], basename( __FILE__ ) ) ) {
            return;
        }
        */

        // Define meta key
        $meta_key = 'lp_' . strtolower( THEMENAME ) . '_options';
        $metaboxes = false;

        // Depending on the Post Type this Content Group will generate, define the metaboxes to save data for
        switch ( $settings['type'] ) {

            case 'lp-reviews':
                $meta_value = array();
                foreach ( $reviews_options as $metabox ) {
                    $meta_value[ $metabox['id'] ] = ( isset( $request[ $metabox['id']] ) ? $request[ $metabox['id'] ] : '' );
                }

                // Update and exit
                update_post_meta( $group_id, $meta_key, $meta_value );  
                return;
                break;

            case 'lp-ads':
                $metaboxes = $ads_options;
                break;

            case 'listing':
                $metaboxes = $listingpro_settings;
                break;

            case 'lp-claims':
                $metaboxes = $claim_options;
                break;

        }

        // Bail if no metaboxes to save meta for
        if ( ! $metaboxes ) {
            return;
        }

        // Iterate through metaboxes, building meta values array and saving it to the Group
        foreach ( $metaboxes as $metabox ) {
            $meta_value[ $metabox['id'] ] = ( isset( $request[ $metabox['id'] ] ) ? $request[ $metabox['id'] ] : '' );
        }
        update_post_meta( $group_id, $meta_key, $meta_value );

        // Update Form Fields Meta
        if ( isset( $request['lp_form_fields_inn'] ) ){
            $fields = array_merge(
                $request['lp_form_fields_inn'],
                lp_save_extra_fields_in_listing( $request['lp_form_fields_inn'], $group_id )
            );
        } else {
            $fields ='';
        }
        update_post_meta( $group_id, 'lp_' . strtolower( THEMENAME ) . '_options_fields', $fields );

    }

    /**
     * Saves ListingPro Events metadata from any metaboxes displayed on the Content Group,
     * emulating ListingPro's save_event_metas() function
     *
     * @since   2.8.6
     *
     * @param   int     $group_id   Group ID
     * @param   array   $settings   Group Settings
     * @param   array   $request    $_REQUEST data, unsanitized
     */
    public function save_event( $group_id, $settings, $request ) {

        // Bail if ListingPro isn't active
        if ( ! function_exists( 'is_plugin_active' ) || ! is_plugin_active( 'listingpro-plugin/plugin.php' ) ) {
            return;
        }

        // Bail if the Content Group isn't for Events
        if ( $settings['type'] != 'events' ) {
            return;
        }

        $event_date = strtotime( sanitize_text_field($request['event_date']) );
        $event_time = sanitize_text_field($request['event_time']);
        $event_loc = sanitize_text_field($request['event_loc']) ;
        $event_lat = sanitize_text_field($request['event_lat']) ;
        $event_lon = sanitize_text_field($request['event_lon']) ;
        $event_ticket_url =sanitize_text_field( $request['event_ticket_url']);
        $eLID   =   sanitize_text_field($request['event-lsiting-id']);

        $get_current_event_ids  =   get_post_meta( $eLID, 'event_id', true );
        if ( isset( $get_current_event_ids ) && is_array( $get_current_event_ids ) ) {
            $get_current_event_ids[]    =   $post_id;
        }
        if ( isset( $get_current_event_ids ) && !is_array( $get_current_event_ids ) ){
            $get_current_event_ids = (array)$get_current_event_ids;
            $get_current_event_ids[]    =   $post_id;
        }

        update_post_meta( $eLID, 'event_id', $get_current_event_ids );

        update_post_meta( $group_id, 'event-lsiting-id', $eLID );
        update_post_meta( $group_id, 'event-date', $event_date );
        update_post_meta( $group_id, 'event-time', $event_time );
        update_post_meta( $group_id, 'event-loc', $event_loc );
        update_post_meta( $group_id, 'event-lat', $event_lat );
        update_post_meta( $group_id, 'event-lon', $event_lon );
        update_post_meta( $group_id, 'ticket-url', $event_ticket_url );

    }

}