<?php
/**
 * Provides compatibility with Popup Maker
 * 
 * @package  Page Generator Pro
 * @author   Tim Carr
 * @version  3.4.1
 */
class Page_Generator_Pro_Popup_Maker extends Page_Generator_Pro_Integration {

    /**
     * Holds the base object.
     *
     * @since   3.4.1
     *
     * @var     object
     */
    public $base;

    /**
     * Constructor
     * 
     * @since   3.4.1
     *
     * @param   object $base    Base Plugin Class
     */
    public function __construct( $base ) {

        // Store base class
        $this->base = $base;

        // Set Plugin
        $this->plugin_folder_filename = 'popup-maker/popup-maker.php';

        // Set Meta Keys used by this Plugin
        $this->meta_keys = array(
            'popup_title',
            'popup_settings',
        );

        // Content Groups: Register Plugin JS
        add_action( 'page_generator_pro_admin_admin_scripts_css', array( $this, 'admin_scripts_css' ), 10, 3 );

        // Content Groups: Register Plugin Metaboxes
        add_action( 'page_generator_pro_groups_ui_add_meta_boxes', array( $this, 'add_meta_boxes' ) );

        // Conditionally display metaboxes based on the Post Type chosen for Generation
        add_filter( 'page_generator_pro_groups_ui_get_post_type_conditional_metaboxes', array( $this, 'get_post_type_conditional_metaboxes' ) );

        // Content Groups: Save Metabox Data
        add_action( 'save_post', array( $this, 'save' ), 10, 2 );

        // Content Groups: Add Overwrite Section if Plugin enabled
        add_filter( 'page_generator_pro_common_get_content_overwrite_sections', array( $this, 'add_overwrite_section' ) );

        // Content Groups: Ignore Plugin meta keys if overwriting is disabled for Plugin
        add_filter( 'page_generator_pro_generate_set_post_meta_ignored_keys', array( $this, 'prevent_post_meta_copy_to_generated_content' ), 10, 4 );

        // Remove Plugin data from Group Settings if Plugin isn't active on Generation
        add_filter( 'page_generator_pro_groups_get_settings_remove_orphaned_settings', array( $this, 'remove_orphaned_settings' ) );

        // Content Groups: Change Cookie ID from Content Group ID to Generated Popup ID
        add_filter( 'page_generator_pro_generate_set_post_meta_popup_settings', array( $this, 'generate_change_cookie_id' ), 10, 5 );

    }

    /**
     * Enqueues CSS and JS
     *
     * @since   3.4.1
     *
     * @param   array       $screen     Screen (screen, section)
     * @param   WP_Post     $post       WordPress Post
     * @param   bool        $minified   Whether to load minified JS
     */
    public function admin_scripts_css( $screen, $post, $minified ) {

    	// Bail if Plugin isn't active
        if ( ! $this->is_active() ) {
            return;
        }

    	// Bail if not editing a Content Group
        if ( $screen['screen'] != 'content_groups' ) {
        	return;
        }
        if ( $screen['section'] != 'edit' ) {
        	return;
        }

    	// Enqueue JS
        wp_enqueue_script( 'pum-admin-popup-editor' );

    }

    /**
     * Adds Plugin Meta Boxes to Content Groups
     *
     * @since   3.4.1
     */
    public function add_meta_boxes() {

        // Bail if Plugin isn't active
        if ( ! $this->is_active() ) {
            return;
        }

        // Register Metabox
        add_meta_box( 'pum_popup_settings', __( 'Popup Settings', 'popup-maker' ), array( $this, 'render_settings_meta_box' ), $this->base->get_class( 'post_type' )->post_type_name, 'normal', 'high' );
        
    }

    /**
     * Define metaboxes that should only display based on the value of Publish > Post Type
     * in the Content Groups UI.
     *
     * @since   3.4.1
     *
     * @param   array   $metaboxes  Metabox ID Keys and Post Type Values array
     * @return  array               Metabox ID Keys and Post Type Values array
     */
    public function get_post_type_conditional_metaboxes( $metaboxes ) {

        return array_merge( $metaboxes, array(
            'pum_popup_settings' => array(
                'popup',
            ),
        ) );

    }

    /**
     * Render the Settings Meta Box
     * 
     * @since 	3.4.1
     * 
     * @param 	int 	$post_id 	Post ID
     * @param 	WP_Post $post 		Post
     */
    public function render_settings_meta_box() {

		global $post;

		// Get settings directly from Post Meta
		$settings = get_post_meta( $post->ID, 'popup_settings', true );

		wp_nonce_field( basename( __FILE__ ), 'pum_popup_settings_nonce' );
		wp_enqueue_script( 'popup-maker-admin' );
		?>
		<script type="text/javascript">
            window.pum_popup_settings_editor = <?php echo PUM_Utils_Array::safe_json_encode( apply_filters( 'pum_popup_settings_editor_var', array(
			'form_args'             => array(
				'id'       => 'pum-popup-settings',
				'tabs'     => PUM_Admin_Popups::tabs(),
				'sections' => PUM_Admin_Popups::sections(),
				'fields'   => PUM_Admin_Popups::fields(),
			),
			'conditions'            => PUM_Conditions::instance()->get_conditions(),
			'conditions_selectlist' => PUM_Conditions::instance()->dropdown_list(),
			'triggers'              => PUM_Triggers::instance()->get_triggers(),
			'cookies'               => PUM_Cookies::instance()->get_cookies(),
			'current_values'        => PUM_Admin_Popups::render_form_values( $settings ),
		) ) ); ?>;
		</script>

		<div id="pum-popup-settings-container" class="pum-popup-settings-container">
			<div class="pum-no-js" style="padding: 0 12px;">
				<p><?php printf( esc_html__( 'If you are seeing this, the page is still loading or there are Javascript errors on this page. %sView troubleshooting guide%s', 'popup-maker' ), '<a href="https://docs.wppopupmaker.com/article/373-checking-for-javascript-errors" target="_blank">', '</a>' ); ?></p>
			</div>
		</div>
		<?php
	
    }

    /**
     * Save Popup Settings to Content Group
     * 
     * @since 	3.4.1
     * 
     * @param 	int 	$post_id 	Post ID
     * @param 	WP_Post $post 		Post
     */
    public function save( $post_id, $post ) {

    	// Bail if Plugin isn't active
        if ( ! $this->is_active() ) {
            return;
        }

        // Bail if nonce fails
        if ( ! isset( $_POST['pum_popup_settings_nonce'] ) || ! wp_verify_nonce( $_POST['pum_popup_settings_nonce'], basename( __FILE__ ) ) ) {
			return;
		}

		// Update Settings
		$settings = ! empty( $_POST['popup_settings'] ) ? $_POST['popup_settings'] : array();

		// Sanitize JSON values.
		$settings['conditions'] = isset( $settings['conditions'] ) ? PUM_Admin_Popups::sanitize_meta( $settings['conditions'] ) : array();
		$settings['triggers']   = isset( $settings['triggers'] ) ? PUM_Admin_Popups::sanitize_meta( $settings['triggers'] ) : array();
		$settings['cookies']    = isset( $settings['cookies'] ) ? PUM_Admin_Popups::sanitize_meta( $settings['cookies'] ) : array();

		// Filter settings
		$settings = apply_filters( 'pum_popup_setting_pre_save', $settings, $post->ID );

		// Sanitize and parse
		$settings = PUM_Admin_Popups::sanitize_settings( $settings );
		$settings = PUM_Admin_Popups::parse_values( $settings );

		// Save settings directly to Post Meta
		update_post_meta( $post_id, 'popup_title', $post->post_title );
		update_post_meta( $post_id, 'popup_settings', $settings );
	
    }
      
    /**
     * Defines available content overwrite sections.
     *
     * @since   3.4.1
     *
     * @param   array   $sections    Content Overwrite Sections
     * @return  array                Content Overwrite Sections
     */
    public function add_overwrite_section( $sections ) {

        // Bail if Plugin isn't active
        if ( ! $this->is_active() ) {
            return $sections;
        }

        // Add Plugin
        $sections['popup_maker'] = __( 'Popup Maker', 'page-generator-pro' );

        // Return
        return $sections;

    }

    /**
     * Adds Plugin Post Meta Keys to the array of excluded Post Meta Keys if Plugin
     * metadata should not be overwritten on Content Generation
     *
     * @since   3.4.1
     *
     * @param   array   $ignored_keys   Ignored Keys (preg_match() compatible regex expressions are supported)
     * @param   int     $post_id        Generated Post ID
     * @param   array   $settings       Group Settings
     * @param   array   $post_args      wp_insert_post() / wp_update_post() arguments
     * @return  array   $ignored_keys   Ignored Keys
     */
    public function prevent_post_meta_copy_to_generated_content( $ignored_keys, $post_id, $settings, $post_args ) {

        // Determine if we want to create/replace Popup Maker metdata
        $overwrite = ( isset( $post_args['ID'] ) && ! array_key_exists( 'popup_maker', $settings['overwrite_sections'] ) ? false : true ); 

        // If overwriting is enabled, no need to exclude anything
        if ( $overwrite ) {
            return $ignored_keys;
        }

        // Add Plugin Meta Keys so they are not overwritten on the Generated Post
        return array_merge( $ignored_keys, $this->meta_keys );

    }

    /**
     * Removes orphaned Plugin metadata in the Group Settings during Generation,
     * if Plugin is not active
     * 
     * @since   3.4.1
     * 
     * @param   array   $settings   Group Settings
     * @return  array               Group Settings
     */
    public function remove_orphaned_settings( $settings ) {

        // Don't remove settings if the Plugin is active
        if ( $this->is_active() ) {
            return $settings;
        }

        // Remove Plugin Meta Keys from the Group Settings during Generation
        return $this->remove_orphaned_settings_metadata( $settings, $this->meta_keys );

    }

    /**
     * Changes the Cookie ID within the Popup Settings from the Content Group ID
     * to the Generated Popup ID
     * 
     * @since 	3.4.1
     * 
     * @param   mixed   $meta_value Meta Value
     * @param 	int 	$post_id 	Generated Post ID
     * @param 	int 	$group_id 	Group ID
     * @param 	array 	$settings 	Group Settings
     * @param 	array 	$post_args 	wp_insert_post() / wp_update_post() arguments
     */
    public function generate_change_cookie_id( $meta_value, $post_id, $group_id, $settings, $post_args ) {

    	// Bail if no meta value
    	if ( ! is_array( $meta_value['triggers'] ) ) {
    		return $meta_value;
    	}

    	// Change Group ID to Generated Popup ID for Cookie Names within Triggers
    	if ( isset( $meta_value['triggers'] ) ) {
    		foreach ( $meta_value['triggers'] as $trigger_index => $trigger ) {
    			foreach ( $trigger['settings']['cookie_name'] as $cookie_index => $cookie_name ) {
    				$meta_value['triggers'][ $trigger_index ]['settings']['cookie_name'][ $cookie_index ] = str_replace( $group_id, $post_id, $cookie_name );
    			}
    		}
    	}

    	// Change Group ID to Generated Popup ID for Cookie Names
    	if ( isset( $meta_value['cookies'] ) ) {
    		foreach ( $meta_value['cookies'] as $cookie_index => $cookie ) {
    			$meta_value['cookies'][ $cookie_index ]['settings']['name'] = str_replace( $group_id, $post_id, $meta_value['cookies'][ $cookie_index ]['settings']['name'] );
    		}
    	}

    	return $meta_value;

    }

 }