<?php
/**
 * SEOPress Integration
 * 
 * @package  Page Generator Pro
 * @author   Tim Carr
 * @version  3.3.7
 */
class Page_Generator_Pro_SEOPress extends Page_Generator_Pro_Integration {

    /**
     * Holds the base object.
     *
     * @since   3.3.7
     *
     * @var     object
     */
    public $base;

    /**
     * Constructor
     * 
     * @since   3.3.7
     *
     * @param   object $base    Base Plugin Class
     */
    public function __construct( $base ) {

        // Store base class
        $this->base = $base;

        // Set Plugin
        $this->plugin_folder_filename = 'wp-seopress/seopress.php';

        // Set Meta Keys used by this Plugin
        $this->meta_keys = array(
            "/^_seopress_(.*)/i",
        );

        // Add Overwrite Section if SEOPress enabled
        add_filter( 'page_generator_pro_common_get_content_overwrite_sections', array( $this, 'add_overwrite_section' ) );

        // Ignore SEOPress meta keys if overwriting is disabled for SEOPress
        add_filter( 'page_generator_pro_generate_set_post_meta_ignored_keys', array( $this, 'prevent_post_meta_copy_to_generated_content' ), 10, 4 );
    
        // Remove Plugin data from Group Settings if Plugin isn't active on Generation
        add_filter( 'page_generator_pro_groups_get_settings_remove_orphaned_settings', array( $this, 'remove_orphaned_settings' ) );

    }

    /**
     * Defines available content overwrite sections.
     *
     * @since   3.3.7
     *
     * @param   array   $sections    Content Overwrite Sections
     * @return  array                Content Overwrite Sections
     */
    public function add_overwrite_section( $sections ) {

        // Bail if SEOPress isn't active
        if ( ! $this->is_active() ) {
            return $sections;
        }

        // Add SEOPress
        $sections['seopress'] = __( 'SEOPress', 'page-generator-pro' );

        // Return
        return $sections;

    }

    /**
     * Adds SEOPress Post Meta Keys to the array of excluded Post Meta Keys if SEOPress
     * metadata should not be overwritten on Content Generation
     *
     * @since   3.3.7
     *
     * @param   array   $ignored_keys   Ignored Keys (preg_match() compatible regex expressions are supported)
     * @param   int     $post_id        Generated Post ID
     * @param   array   $settings       Group Settings
     * @param   array   $post_args      wp_insert_post() / wp_update_post() arguments
     * @return  array   $ignored_keys   Ignored Keys
     */
    public function prevent_post_meta_copy_to_generated_content( $ignored_keys, $post_id, $settings, $post_args ) {

        // Determine if we want to create/replace SEOPress metdata
        $overwrite = ( isset( $post_args['ID'] ) && ! array_key_exists( 'seopress', $settings['overwrite_sections'] ) ? false : true ); 

        // If overwriting is enabled, no need to exclude anything
        if ( $overwrite ) {
            return $ignored_keys;
        }

        // Add SEOPress Meta Keys so they are not overwritten on the Generated Post
        return array_merge( $ignored_keys, $this->meta_keys );

    }

    /**
     * Removes orphaned SEOPress metadata in the Group Settings during Generation,
     * if SEOPress is not active
     * 
     * @since   3.3.7
     * 
     * @param   array   $settings   Group Settings
     * @return  array               Group Settings
     */
    public function remove_orphaned_settings( $settings ) {

        // Don't remove settings if the Plugin is active
        if ( $this->is_active() ) {
            return $settings;
        }

        // Remove SEOPress Meta Keys from the Group Settings during Generation
        return $this->remove_orphaned_settings_metadata( $settings, $this->meta_keys );

    }

 }