<?php
/**
 * Keywords Source CSV URL class, used to read data
 * from a supplied CSV URL which is then used as a Keyword
 * 
 * @package Page_Generator_Pro
 * @author  Tim Carr
 * @version 3.0.8
 */
class Page_Generator_Pro_Keywords_Source_CSV_URL {

    /**
     * Holds the base object.
     *
     * @since   3.0.8
     *
     * @var     object
     */
    public $base;

    /**
     * Constructor
     * 
     * @since   3.0.8
     *
     * @param   object $base    Base Plugin Class
     */
    public function __construct( $base ) {

        // Store base class
        $this->base = $base;

        // Register this Keyword Source
        add_filter( 'page_generator_pro_keywords_register_sources', array( $this, 'register' ) );

        // Define parameters for the Keyword before saving
        add_filter( 'page_generator_pro_keywords_save_csv_url', array( $this, 'save' ) );

        // Refresh Keyword Terms before starting generation
        add_filter( 'page_generator_pro_keywords_refresh_terms_csv_url', array( $this, 'refresh_terms' ), 10, 2 );

    }

    /**
     * Returns the programmatic name of the source
     *
     * @since   3.0.8
     *
     * @return  string
     */
    public function get_name() {

        return 'csv_url';

    }

    /**
     * Returns the label of the source
     *
     * @since   3.0.8
     *
     * @return  string
     */
    public function get_label() {

        return __( 'CSV URL', 'page-generator-pro' );

    }

    /**
     * Registers this Source with the Keyword Sources system, so it's available
     * to Keywords
     *
     * @since   3.0.8
     *
     * @param   array   $sources    Sources
     * @return  array               Sources
     */
    public function register( $sources ) {

        return array_merge( $sources, array(
            $this->get_name() => array(
                'name'      => $this->get_name(),
                'label'     => $this->get_label(),
                'options'   => array(
                    'url' => array(
                        'type'          => 'url',
                        'label'         => __( 'CSV URL', 'page-generator-pro' ),
                        'description'   => __( 'The CSV File URL to use as the Terms for this Keyword', 'page-generator-pro' ),
                    ),
                    'columns_first_row' => array(
                        'type'          => 'toggle',
                        'label'         => __( 'Columns in First Row', 'page-generator-pro' ),
                        'file_type'     => 'text/csv',
                        'description'   => __( 'If enabled, the first row\'s values will be used as column names', 'page-generator-pro' ),
                    ),
                    'delimiter'     => array(
                        'type'          => 'text',
                        'label'         => __( 'Delimiter', 'page-generator-pro' ),
                        'description'   => __( 'If "Columns in First Row" is enabled, specify the delimiter used between columns here. Typically this will be a comma.', 'page-generator-pro' ),
                    ),
                    'preview'       => array(
                        'type'          => 'preview',
                        'label'         => __( 'Terms', 'page-generator-pro' ),
                    ),
                ),
            ),
        ) );

    }

    /**
     * Prepares Keyword Data for this Source, based on the supplied form data,
     * immediately before it's saved to the Keywords table in the database
     *
     * @since   3.0.8
     *
     * @param   array   $keyword        Keyword Parameters
     * @return  mixed                   WP_Error | Keyword Parameters
     */
    public function save( $keyword ) {

        // Specify columns_first_row option
        if ( ! isset( $keyword['options']['columns_first_row'] ) ) {
            $keyword['options']['columns_first_row'] = 0;
            $keyword['options']['delimiter'] = '';
        }

        // If the URL is for Google Sheets, change the output format to CSV
        // if this hasn't already been done
        if ( strpos( $keyword['options']['url'], 'docs.google.com' ) ) {
            if ( ! strpos( $keyword['options']['url'], 'export?format=csv' ) ) {
                // Get URL parts
                $url = parse_url( $keyword['options']['url'] );

                // Remove '/edit' if it exists and append /export?format=csv
                $keyword['options']['url'] = $url['scheme'] . '://' . $url['host'] . str_replace( '/edit', '', $url['path'] ) . '/export?format=csv';
            }
        }

        // Get Keyword Terms
        $keywords_terms = $this->get( $keyword );

        // Bail if an error occured
        if ( is_wp_error( $keywords_terms ) ) {
            return $keywords_terms;
        }

        // Merge delimiter, columns and data with Keyword
        $keyword = array_merge( $keyword, array(
            'delimiter' => $keyword['options']['delimiter'],
            'columns'   => ( is_array( $keywords_terms[ $keyword['keyword'] ]['columns'] ) ? implode( ',', $keywords_terms[ $keyword['keyword'] ]['columns'] ) : '' ),
            'data'      => implode( "\n", $keywords_terms[ $keyword['keyword'] ]['data'] ),
        ) );

        return $keyword;

    }

    /**
     * Refresh the given Keyword's Columns and Terms by fetching them from the CSV file
     * immediately before starting generation.
     *
     * @since   3.0.8
     *
     * @param   string  $terms      Terms
     * @param   array   $keyword    Keyword
     * @return  mixed               WP_Error | array (delimiter,columns,data)
     */
    public function refresh_terms( $terms, $keyword ) {

        // Get Keyword Terms
        $keywords_terms = $this->get( $keyword );

        // Bail if an error occured
        if ( is_wp_error( $keywords_terms ) ) {
            return $keywords_terms;
        }

        // Return Keyword
        return $keywords_terms[ $keyword['keyword'] ];

    }

    /**
     * Fetches Terms from the CSV URL, based on the Keyword settings
     *
     * @since   3.0.8
     *
     * @param   array   $keyword    Keyword
     * @return  mixed               WP_Error | array (delimiter,columns,data)
     */
    private function get( $keyword ) {

        // Read CSV URL  with or without columns depending on the Keyword settings
        if ( $keyword['options']['columns_first_row'] ) {
            return $this->base->get_class( 'keywords' )->read_csv_file( 
                $keyword['options']['url'],
                'columns_single_keyword',
                $keyword['options']['delimiter'],
                $keyword['keyword']
            );
        }

        return $this->base->get_class( 'keywords' )->read_csv_file( 
            $keyword['options']['url'],
            false,
            false,
            $keyword['keyword']
        );

    }

}